@@ -98,7 +98,7 @@ $::SIG{TERM} = 'IGNORE';
 $::SIG{PIPE} = 'IGNORE';
 
 # Contains the version string for the current running Bugzilla.
-$::param{'version'} = '2.16.5';
+$::param{'version'} = '2.16.6';
 
 $::dontchange = "--do_not_change--";
 $::chooseone = "--Choose_one:--";
@@ -1566,7 +1566,7 @@ $::template ||= Template->new(
         js => sub
         {
             my ($var) = @_;
-            $var =~ s/([\\\'\"])/\\$1/g; 
+            $var =~ s/([\\\'\"\/])/\\$1/g; 
             $var =~ s/\n/\\n/g; 
             $var =~ s/\r/\\r/g; 
             return $var;
